@@ -434,8 +434,8 @@ class Searcher(Node):
 
     def in_faces(self, tensors, delta=-1, assign_multiple=False):
         """
-        TODO:
-            * check rare case of point+-delta outside box
+        TODO-0:
+            * check case of point+-delta outside box!
 
         Examples:
             >>> import tfields
@@ -451,20 +451,20 @@ class Searcher(Node):
             >>> assert np.array_equal(box_res, usual_res)
 
         """
-        # raise ValueError("Broken feature. We are working on it!")
         if not self.is_root():
             raise ValueError("in_faces may only be called by root Node.")
-        if self.at_intersection != "keep":
-            raise ValueError(
-                "Intersection method must be 'keep' for in_faces" "method."
-            )
-
-        if self.mesh.nfaces() == 0:
-            return np.empty((tensors.shape[0], 0), dtype=bool)
+        assert self.at_intersection == "keep", "Intersection method must be 'keep'"
+        assert not assign_multiple, "Not yet implemented. Does interfere with delta."
         if delta == -1:
             delta = self.delta
 
-        masks = np.zeros((len(tensors), self.mesh.nfaces()), dtype=bool)
+        indices = np.full(tensors.shape[0], -1, dtype=int)
+        if self.mesh.nfaces() == 0:
+            if assign_multiple:
+                return [[-1] * len(indices)]
+            else:
+                return indices
+
         with tensors.tmp_transform(self.mesh.coord_sys):
             for i, point in enumerate(iter(tensors)):
                 leaf = self.find_leaf(point)
@@ -474,10 +474,10 @@ class Searcher(Node):
                     continue
                 leaf_mask = leaf.template.triangles()._in_triangles(point, delta)
                 original_face_indices = leaf.template.maps[3].fields[0][leaf_mask]
-                if not assign_multiple and len(original_face_indices) > 0:
-                    original_face_indices = original_face_indices[:1]
-                masks[i, original_face_indices] = True
-        return masks
+                if len(original_face_indices):
+                    indices[i] = original_face_indices[0]
+                    # if len(original_face_indices) > 1 -> assign_multiple_faces
+        return indices
 
 
 if __name__ == "__main__":
