@@ -1,3 +1,4 @@
+import typing
 import numpy as np
 import functools
 import tfields.lib.util
@@ -92,7 +93,7 @@ def igrid(*base_vectors, **kwargs):
         ...                        [ 1.,  4.,  7.]])
 
     """
-    iter_order = kwargs.pop('iter_order', np.arange(len(base_vectors)))
+    iter_order = kwargs.pop("iter_order", np.arange(len(base_vectors)))
     base_vectors = ensure_complex(*base_vectors)
 
     # create the grid
@@ -103,9 +104,12 @@ def igrid(*base_vectors, **kwargs):
     else:
         base_vectors = to_base_vectors(*base_vectors)
 
-        obj = np.empty(shape=(functools.reduce(lambda x, y: x * y,
-                                               map(len, base_vectors)),
-                              len(base_vectors)))
+        obj = np.empty(
+            shape=(
+                functools.reduce(lambda x, y: x * y, map(len, base_vectors)),
+                len(base_vectors),
+            )
+        )
 
         def loop_rec(y, n_max, i=0, n=None, *vals):
             if n is None:
@@ -173,6 +177,7 @@ def swap_columns(array, *index_tuples):
     Args:
         array (list or array)
         expects tuples with indices to swap as arguments
+
     Examples:
         >>> import tfields
         >>> l = np.array([[3, 2, 1, 0], [6, 5, 4, 3]])
@@ -180,15 +185,13 @@ def swap_columns(array, *index_tuples):
         >>> l
         array([[0, 1, 2, 3],
                [3, 4, 5, 6]])
-
     """
     # test swap_indices type
     for si in index_tuples:
-        if hasattr(si, '__iter__'):
+        if hasattr(si, "__iter__"):
             if len(si) == 2:
                 continue
-        raise TypeError("swap_indices must be tuple but is {}"
-                        .format(si))
+        raise TypeError("swap_indices must be tuple but is {}".format(si))
     for i, j in index_tuples:
         array[:, [i, j]] = array[:, [j, i]]
 
@@ -198,13 +201,13 @@ def swap_rows(array, *args):
     Args:
         array (list)
         expects tuples with indices to swap as arguments
+
     Examples:
         >>> import tfields
         >>> l = [[3,3,3], [2,2,2], [1,1,1], [0, 0, 0]]
         >>> tfields.lib.grid.swap_rows(l, (1, 2), (0, 3))
         >>> l
         [[0, 0, 0], [1, 1, 1], [2, 2, 2], [3, 3, 3]]
-
     """
     for i, j in args:
         array[i], array[j] = array[j], array[i]
@@ -212,7 +215,8 @@ def swap_rows(array, *args):
 
 def compare_permutations(permut1, permut2):
     """
-    Return what you need to switch in order to make permut1 become permut2
+    Return what rows you need to switch in order to make permut1 become permut2
+
     Examples:
         >>> import tfields
         >>> a = [1, 2, 0, 4, 3]
@@ -225,7 +229,6 @@ def compare_permutations(permut1, permut2):
         [0, 1, 2, 3, 4]
         >>> a == b
         True
-
     """
     swap_indices = []
     permut1 = list(permut1)
@@ -239,6 +242,28 @@ def compare_permutations(permut1, permut2):
     return swap_indices
 
 
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
+def change_iter_order(
+    bv_lengths: typing.List[int],
+    iter_order: typing.List[int],
+    iter_order_new: typing.List[int],
+):
+    """
+    Args:
+        bv_lengths: lenghts of basis vectors
+
+
+    Returns:
+        indices for changeing fields generated with iter_order to iter_order_new
+    """
+    indices = np.arange(np.prod(bv_lengths))
+    shape = []
+    for i in iter_order:
+        shape.append(bv_lengths[i])
+    indices = indices.reshape(shape)
+    # swaps = compare_permutations(iter_order_new, iter_order)
+    swaps = compare_permutations(iter_order, iter_order_new)
+    # swaps = compare_permutations(iter_order_new, np.arange(len(iter_order)))
+    for swap in swaps:
+        indices = np.swapaxes(indices, swap[0], swap[1])
+    # indices = np.transpose(indices, axes=axes_transp)
+    return np.array(indices.flat)
