@@ -9,7 +9,6 @@ Triangulated mesh class and methods
 import logging
 import os
 import numpy as np
-import rna
 import tfields
 
 # obj imports
@@ -534,25 +533,21 @@ class Mesh3D(tfields.TensorMaps):
     def nfaces(self):
         return self.faces.shape[0]
 
-    def in_faces(self, points, delta, assign_multiple=False):
+    def in_faces(self, points, delta, **kwargs):
         """
         Check whether points lie within triangles with Barycentric Technique
-        see Triangles3D.in_triangles
-        If multiple requests are done on huge meshes,
+        see Triangles3D.in_triangles. If multiple requests are done on huge meshes,
         this can be hugely optimized by requesting the property
-        self.tree or setting it to self.tree = <saved tree> before
-        calling in_faces
+        self.tree or setting it to self.tree = <saved tree> before calling in_faces.
         """
         key = "mesh_tree"
         if hasattr(self, "_cache") and key in self._cache:
             log = logging.getLogger()
             log.info("Using cached decision tree to speed up point - face mapping.")
-            masks = self.tree.in_faces(points, delta, assign_multiple=assign_multiple)
+            indices = self.tree.in_faces(points, delta, **kwargs)
         else:
-            masks = self.triangles().in_triangles(
-                points, delta, assign_multiple=assign_multiple
-            )
-        return masks
+            indices = self.triangles().in_triangles(points, delta, **kwargs)
+        return indices
 
     @property
     def tree(self):
@@ -561,15 +556,20 @@ class Mesh3D(tfields.TensorMaps):
         hugely optimize 'in_faces' searches
 
         Examples:
-            # >>> mesh = tfields.Mesh3D.grid((0, 1, 3), (1, 2, 3), (2, 3, 3))
-            # >>> _ = mesh.tree
-            # >>> assert hasattr(mesh, '_cache')
-            # >>> assert 'mesh_tree' in mesh._cache
-            # >>> mask = mesh.in_faces(tfields.Points3D([[0.2, 1.2, 2.0]]),
-            # ...                      0.00001)
-            # >>> assert mask.sum() == 1  # one point in one triangle
+            >>> import numpy as np
+            >>> import tfields
+            >>> mesh = tfields.Mesh3D.grid((0, 1, 3), (1, 2, 3), (2, 3, 3))
+            >>> _ = mesh.tree
+            >>> assert hasattr(mesh, '_cache')
+            >>> assert 'mesh_tree' in mesh._cache
+            >>> face_indices = mesh.in_faces(tfields.Points3D([[0.2, 1.2, 2.0]]),
+            ...                      0.00001)
+
+            You might want to know the number of points per face
+            >>> unique, counts = np.unique(face_indices, return_counts=True)
+            >>> dict(zip(unique, counts))  # one point on triangle number 16
+            {16: 1}
         """
-        raise ValueError("Broken feature. We are working on it!")
         if not hasattr(self, "_cache"):
             self._cache = {}
         key = "mesh_tree"
@@ -615,15 +615,11 @@ class Mesh3D(tfields.TensorMaps):
         TODO:
             fields template not yet implemented
         """
-        face_indices = np.arange(self.maps[3].shape[0])
         cents = tfields.Tensors(sub_mesh.centroids())
-        mask = self.in_faces(cents, delta=None)
+        face_indices = self.in_faces(cents, delta=None)
         inst = sub_mesh.copy()
         if inst.maps:
-            scalars = []
-            for face_mask in mask:
-                scalars.append(face_indices[face_mask][0])
-            inst.maps[3].fields = [tfields.Tensors(scalars, dim=1)]
+            inst.maps[3].fields = [tfields.Tensors(face_indices, dim=1)]
         else:
             inst.maps = [
                 tfields.TensorFields([], tfields.Tensors([], dim=1), dim=3, dtype=int)
@@ -639,7 +635,7 @@ class Mesh3D(tfields.TensorMaps):
         return_point_face_assignment=False,
     ):
         """
-        project the points of the tensor_field to a copy of the mesh
+        Project the points of the tensor_field to a copy of the mesh
         and set the face values accord to the field to the maps field.
         If no field is present in tensor_field, the number of points in a mesh
         is counted.
@@ -662,14 +658,15 @@ class Mesh3D(tfields.TensorMaps):
             ...                           [1, 2, 3, 4])
             >>> m = tfields.Mesh3D([[0,0,0], [1,0,0], [1,1,0], [0,1,0], [0,2,0], [1,2,0]],
             ...                     maps=[mp])
-
-            Projecting points onto the mesh gives the count
             >>> points = tfields.Tensors([[0.5, 0.2, 0.0],
             ...                           [0.5, 0.02, 0.0],
             ...                           [0.5, 0.8, 0.0],
             ...                           [0.5, 0.8, 0.1]])  # not contained
+
+            Projecting points onto the mesh gives the count
             >>> m_points = m.project(points, delta=0.01)
-            >>> assert m_points.maps[3].fields[0].equal([2, 1, 0, 0])
+            >>> list(m_points.maps[3].fields[0])
+            [2, 1, 0, 0]
 
             TensorFields with arbitrary size are projected,
             combinging the fields automatically
@@ -681,8 +678,7 @@ class Mesh3D(tfields.TensorMaps):
             ...                            [[9, 9]] * 2])]
             >>> tf = tfields.TensorFields(points, *fields)
             >>> m_tf = m.project(tf, delta=0.01)
-            >>> assert m_tf.maps[3].fields[0].equal([2, 42, np.nan, np.nan],
-            ...                                     equal_nan=True)
+            >>> assert m_tf.maps[3].fields[0].equal([2, 42, np.nan, np.nan], equal_nan=True)
             >>> assert m_tf.maps[3].fields[1].equal([[1, 2, 3],
             ...                                      [3, 4, 5],
             ...                                      [np.nan] * 3,
@@ -731,18 +727,9 @@ class Mesh3D(tfields.TensorMaps):
         if point_face_assignment is not None:
             if len(point_face_assignment) != len(tensor_field):
                 raise ValueError("Template needs same lenght as tensor_field")
-            point_face_assignment = point_face_assignment
         else:
-            mask = self.in_faces(tensor_field, delta=delta)
+            point_face_assignment = self.in_faces(tensor_field, delta=delta)
 
-            face_indices = np.arange(n_faces)
-            point_face_assignment = [
-                face_indices[mask[p_index, :]] for p_index in point_indices
-            ]
-            point_face_assignment = np.array(
-                [fa if len(fa) != 0 else [-1] for fa in point_face_assignment]
-            )
-            point_face_assignment = point_face_assignment.reshape(-1)
         point_face_assignment_set = set(point_face_assignment)
 
         # merge the fields according to point_face_assignment
@@ -1168,21 +1155,6 @@ class Mesh3D(tfields.TensorMaps):
                 templates.append(template)
             return parts, templates
 
-    def plot(self, **kwargs):  # pragma: no cover
-        """
-        Forwarding to rna.plotting.plot_mesh
-        """
-        scalars_demanded = (
-            "color" not in kwargs
-            and "facecolors" not in kwargs
-            and any([v in kwargs for v in ["vmin", "vmax", "cmap"]])
-        )
-        map_index = kwargs.pop("map_index", None if not scalars_demanded else 0)
-        if map_index is not None:
-            if not len(self.maps[3]) == 0:
-                kwargs["color"] = self.maps[3].fields[map_index]
-        return rna.plotting.plot_mesh(self, self.faces, **kwargs)
-
 
 if __name__ == "__main__":  # pragma: no cover
     import doctest
