@@ -7,6 +7,7 @@ Mail:       daniel.boeckenhoff@ipp.mpg.de
 part of tfields library
 """
 import warnings
+import typing
 import numpy as np
 import tfields
 import logging
@@ -529,54 +530,7 @@ class Triangles3D(tfields.TensorFields):
             np.array: boolean mask, True where point in a triangle within delta
 
         Examples:
-            >>> m = tfields.Mesh3D([[1,0,0], [0,1,0], [0,0,0]], faces=[[0, 1, 2]]);
-            >>> assert np.array_equal(
-            ...     m.triangles()._in_triangles(tfields.Points3D([[0.2, 0.2, 0]])),
-            ...     np.array([True], dtype=bool))
-
-            All Triangles are tested
-
-            >>> m2 = tfields.Mesh3D([[1,0,0], [0,1,0], [0,0,0], [4,0,0], [4, 4, 0], [8, 0, 0]],
-            ...                     faces=[[0, 1, 2], [3, 4, 5]]);
-
-            >>> assert np.array_equal(
-            ...     m2.triangles()._in_triangles(np.array([0.2, 0.2, 0])),
-            ...     np.array([True, False], dtype=bool))
-            >>> assert np.array_equal(
-            ...     m2.triangles()._in_triangles(np.array([5, 2, 0])),
-            ...     np.array([False,  True], dtype=bool))
-
-            delta allows to accept points that lie within delta orthogonal to the tringle plain
-
-            >>> assert np.array_equal(
-            ...     m2.triangles()._in_triangles(np.array([0.2, 0.2, 9000]), 0.0),
-            ...     np.array([False, False], dtype=bool))
-            >>> assert np.array_equal(
-            ...     m2.triangles()._in_triangles(np.array([0.2, 0.2, 0.1]), 0.2),
-            ...     np.array([ True, False], dtype=bool))
-
-            if you set delta to None, the minimal distance point(s) are accepted
-
-            >>> assert np.array_equal(
-            ...     m2.triangles()._in_triangles(np.array([0.2, 0.2, 0.1]), None),
-            ...     np.array([ True, False], dtype=bool))
-
-            If you define triangles that have colinear side vectors or in general lead to
-            not invertable matrices the you will always get False
-
-            >>> m3 = tfields.Mesh3D([[0,0,0], [2,0,0], [4,0,0], [0,1,0]],
-            ...                     faces=[[0, 1, 2], [0, 1, 3]]);
-            >>> mask = m3.triangles()._in_triangles(np.array([0.2, 0.2, 0]), delta=0.3)
-            >>> assert np.array_equal(mask,
-            ...                       np.array([False,  True], dtype=bool))
-
-        Raises:
-            Wrong format of point will raise a ValueError
-            >>> m.triangles()._in_triangles(tfields.Points3D([[0.2, 0.2, 0], [0.2, 0.2, 0]]))
-            Traceback (most recent call last):
-            ...
-            ValueError: point must be castable to shape 3 but is of shape (2, 3)
-
+            see tests/test_triangles_3d.py
         """
 
         if self.ntriangles() == 0:
@@ -628,7 +582,12 @@ class Triangles3D(tfields.TensorFields):
 
         return np.array(barycentric_bools & orthogonal_acceptance)
 
-    def in_triangles(self, tensors, delta=0.0, assign_multiple=False):
+    def in_triangles(
+        self,
+        tensors,
+        delta: typing.Optional[float] = 0.0,
+        assign_multiple: bool = False,
+    ) -> typing.Union[typing.List[typing.List[int]], np.array]:
         """
         Barycentric method to obtain, which tensors are containes in any of the triangles
 
@@ -636,61 +595,41 @@ class Triangles3D(tfields.TensorFields):
             tensors (Points3D instance)
 
             optional:
-            delta (:obj:`float` | :obj:`None`, optional):
-                :obj:`float`: normal distance to a triangle, that the points
+            delta:
+                :obj:`float`: Normal distance to a triangle, that the points
                     are concidered to be contained in the triangle.
-                :obj:`None`: find the minimum distance
-                default is 0.
-            assign_multiple (bool): if True, one point may belong to multiple
+                :obj:`None`: Find the minimum distance.
+                Default is 0.
+            assign_multiple: If True, one point may belong to multiple
                 triangles at the same time. In the other case the first
                 occurence will be True the other False
 
         Returns:
-            np.ndarray: 2-d mask which is True, where a point is in a triangle
-                              triangle index (axis = 1)->
-                points index
-                (axis = 0)          1        0       0
-                      |             1        0       0
-                      V             0        0       1
-
-                if assign_multiple == False:
-                    there is just one True for each points index
-                triangle indices can have multiple true values
-
-                For Example, if you want to know the number of points in one
-                face, just sum over all points:
-                >> tris.in_triangles(poits).sum(axis=0)[face_index]
-
+            list: [index(or indices if assign_multiple) of triangle for point in tensors]
         """
+        indices = np.full(tensors.shape[0], -1, dtype=int)
         if self.ntriangles() == 0:
-            return np.empty((tensors.shape[0], 0), dtype=bool)
+            if assign_multiple:
+                return [[-1] * len(indices)]
+            else:
+                return indices
 
-        masks = np.zeros((len(tensors), self.ntriangles()), dtype=bool)
         with tensors.tmp_transform(self.coord_sys):
             for i, point in enumerate(iter(tensors)):
-                masks[i] = self._in_triangles(point, delta)
-
-        if len(masks) == 0:
-            # empty sequence
-            return masks
-
-        if not assign_multiple:
-            """
-            index of first faces containing the point for each point. This gets the
-            first membership index always. ignoring multiple triangle memberships
-            """
-            faceMembershipIndices = np.argmax(masks, axis=1)
-            # True if point lies in any triangle
-            membershipBools = masks.sum(axis=1) != 0
-
-            masks = np.full(masks.shape, False, dtype=bool)
-            for j, valid in enumerate(membershipBools):
-                if valid:
-                    masks[j, faceMembershipIndices[j]] = True
-            """
-            masks is now the matrix as described in __doc__
-            """
-        return masks
+                mask = self._in_triangles(point, delta)
+                if np.any(mask):
+                    if assign_multiple:
+                        index = np.argwhere(mask == np.amax(mask))
+                        index.flatten().tolist()
+                        indices.append(index)
+                    else:
+                        indices[i] = np.argmax(mask)
+                else:
+                    if assign_multiple:
+                        indices.append([-1])
+                    else:
+                        indices[i] = -1
+        return indices
 
     def _on_edges(self, point):
         """
